function [ k, alpha ] = MLEPSDfit( trace, freq )
%[k,alpha] = MLEPSDfit(trace,freq) loads a trace at freq, computes the PSD
% fits the function from Lansdorp-2012 and returns the spring
%constant, k, and dissipation, alpha 

blocks = 30;

%Compute the PSD of the time series:

[PSD, f] = pwelch(trace, floor(length(trace)/blocks),[],[],freq,'twosided');

%Now we will need to do an unconstrained optimization to minimize the MLE
%cost function:

%For the PSD, the shape parameter is the number of blocks used:

eta = blocks;

%We use a least-squres fit to get the approximate parameters used in the
%maximum liklihood calculation:

ffun = fittype('4.1/(2*3.14*a*(k^2/(4*9.87*a^2)+x^2))');
ffun_limited = fitoptions(ffun);
%Setting fit parameters:
set(ffun_limited, 'Lower', [1E-6,0], 'Upper',[1E-4,Inf], 'StartPoint',[1.5E-5,0.03],'MaxFunEvals',6000,'MaxIter',4000);
ffun = fittype('4.1/(2*3.14*a*(k^2/(4*9.87*a^2)+x^2))', 'options', ffun_limited);
fitObject = fit(f, PSD, ffun);

coefficients = coeffvalues(fitObject);
alpha_guess = coefficients(1);
k_guess = coefficients(2);

%Define the anonymous function for the cost function in terms of two other
%functions defined at the end of this file: x = [a,k]

cost_fun = @(x) cost(x(1),x(2),PSD,f,eta, freq);

[fits,~,flag] = fminsearch(cost_fun,[alpha_guess,k_guess]);

%If the minimum search does not find a min., I return an error.  One could
%also handle this, on bad data where this is a common problem, by using the
%least-squares values obtained above

if flag ~= 1
    error('fminsearch minimization of cost function failed to converge');
end;

k = fits(2);
alpha = fits(1);

%disp(sprintf('Alpha: %g; k: %g',[alpha,k]));


%Make a plot the check the quality of the fit:
% for n=1:length(f)
%    PSD_fits(n) = PSD_ideal(alpha,k,f(n),freq);
% end;
% 
% 
% figure;
% loglog(f,PSD,'.k');
% hold on;
% loglog(f,PSD_fits,'-r');





end

%Function for the AV from the Langevin equation:

function [PSD] = PSD_ideal(a,k,f,fs)
kT = 4.1;
PSD = 2*kT*a/k^3*(k+(2*a*fs*sin(pi*f/fs)^2*sinh(k/a/fs))/(cos(2*pi*f/fs)-cosh(k/a/fs)));
%PSD = kT/(2*pi*a*(k^2/(4*pi^2*a^2)+f^2));
end

function [F] = cost(a,k,PSD_measured,f,eta, freq)
for n=1:length(f)
    term(n) = eta*(PSD_measured(n)/PSD_ideal(a,k,f(n),freq)+log(PSD_ideal(a,k,f(n),freq)));  
end;
F = sum(term);
end

